@@ -93,6 +93,13 @@ but you can turn it off or on explicitly, if needed.
 --silent
 type=bool-set
 Do not print out anything to stdout during operation.
+
+
+--z-index -z
+default=0
+Z-index of the image. When negative, text will be displayed on top of the image. Use
+a double minus for values under the threshold for drawing images under cell background
+colors. For example, --1 evaluates as -1,073,741,825.
 '''
 
 
@@ -182,8 +189,8 @@ def write_chunked(cmd, data):
         cmd.clear()
 
 
-def show(outfile, width, height, fmt, transmit_mode='t', align='center', place=None):
-    cmd = {'a': 'T', 'f': fmt, 's': width, 'v': height}
+def show(outfile, width, height, zindex, fmt, transmit_mode='t', align='center', place=None):
+    cmd = {'a': 'T', 'f': fmt, 's': width, 'v': height, 'z': zindex}
     if place:
         set_cursor_for_place(place, cmd, width, height, align)
     else:
@@ -201,6 +208,14 @@ def show(outfile, width, height, fmt, transmit_mode='t', align='center', place=N
         write_chunked(cmd, data)
 
 
+def parse_z_index(val):
+    origin = 0
+    if val.startswith('--'):
+        val = val[1:]
+        origin = -1073741824
+    return origin + int(val)
+
+
 def process(path, args, is_tempfile):
     m = identify(path)
     ss = get_screen_size()
@@ -219,7 +234,7 @@ def process(path, args, is_tempfile):
         fmt = 24 if m.mode == 'rgb' else 32
         transmit_mode = 't'
         outfile, width, height = convert(path, m, available_width, available_height, args.scale_up)
-    show(outfile, width, height, fmt, transmit_mode, align=args.align, place=args.place)
+    show(outfile, width, height, args.z_index, fmt, transmit_mode, align=args.align, place=args.place)
     if not args.place:
         print()  # ensure cursor is on a new line
     return file_removed
@@ -360,6 +375,11 @@ def main(args=sys.argv):
     except Exception:
         raise SystemExit('Not a valid place specification: {}'.format(args.place))
 
+    try:
+        args.z_index = parse_z_index(args.z_index)
+    except Exception:
+        raise SystemExit('Not a valid z-index specification: {}'.format(args.z_index))
+
     if args.detect_support:
         if not detect_support(wait_for=args.detection_timeout, silent=True):
             raise SystemExit(1)
